from .base_message import BaseMessage
from .enums import ChannelType, EncodingType, MessageType, WebhookVersion
from .messenger import (
    MessengerAudio,
    MessengerFile,
    MessengerImage,
    MessengerOptions,
    MessengerResource,
    MessengerText,
    MessengerVideo,
)
from .mms import MmsAudio, MmsImage, MmsResource, MmsVcard, MmsVideo
from .rcs import RcsCustom, RcsFile, RcsImage, RcsResource, RcsText, RcsVideo
from .sms import Sms, SmsOptions
from .viber import (
    ViberAction,
    ViberFile,
    ViberFileOptions,
    ViberFileResource,
    ViberImage,
    ViberImageOptions,
    ViberImageResource,
    ViberText,
    ViberTextOptions,
    ViberVideo,
    ViberVideoOptions,
    ViberVideoResource,
)
from .whatsapp import (
    WhatsappAudio,
    WhatsappAudioResource,
    WhatsappContext,
    WhatsappCustom,
    WhatsappFile,
    WhatsappFileResource,
    WhatsappImage,
    WhatsappImageResource,
    WhatsappSticker,
    WhatsappStickerId,
    WhatsappStickerUrl,
    WhatsappTemplate,
    WhatsappTemplateResource,
    WhatsappTemplateSettings,
    WhatsappText,
    WhatsappVideo,
    WhatsappVideoResource,
)

__all__ = [
    'BaseMessage',
    'ChannelType',
    'EncodingType',
    'MessageType',
    'MessengerAudio',
    'MessengerFile',
    'MessengerImage',
    'MessengerOptions',
    'MessengerResource',
    'MessengerText',
    'MessengerVideo',
    'MmsAudio',
    'MmsImage',
    'MmsResource',
    'MmsVcard',
    'MmsVideo',
    'RcsCustom',
    'RcsFile',
    'RcsImage',
    'RcsResource',
    'RcsText',
    'RcsVideo',
    'Sms',
    'SmsOptions',
    'ViberAction',
    'ViberFile',
    'ViberFileOptions',
    'ViberFileResource',
    'ViberImage',
    'ViberImageOptions',
    'ViberImageResource',
    'ViberText',
    'ViberTextOptions',
    'ViberVideo',
    'ViberVideoOptions',
    'ViberVideoResource',
    'WebhookVersion',
    'WhatsappAudio',
    'WhatsappAudioResource',
    'WhatsappContext',
    'WhatsappCustom',
    'WhatsappFile',
    'WhatsappFileResource',
    'WhatsappImage',
    'WhatsappImageResource',
    'WhatsappSticker',
    'WhatsappStickerId',
    'WhatsappStickerUrl',
    'WhatsappTemplate',
    'WhatsappTemplateResource',
    'WhatsappTemplateSettings',
    'WhatsappText',
    'WhatsappVideo',
    'WhatsappVideoResource',
]



from typing import Optional, Union

from pydantic import BaseModel, Field
from vonage_utils.types import PhoneNumber

from .base_message import BaseMessage
from .enums import ChannelType, MessageType


class MmsResource(BaseModel):
    """Model for a resource in an MMS message.

    Args:
        url (str): The URL of the resource.
        caption (str, Optional): Additional text to accompany the resource.
    """

    url: str
    caption: Optional[str] = Field(None, min_length=1, max_length=2000)


class BaseMms(BaseMessage):
    """Model for a base MMS message.

    Args:
        to (PhoneNumber): The recipient's phone number in E.164 format. Don't use a leading plus sign.
        from_ (Union[PhoneNumber, str]): The sender's phone number in E.164 format. Don't use a leading plus sign.
        ttl (int, Optional): The duration in seconds for which the message is valid.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API will be used to send Status Webhook messages for this particular message.
    """

    to: PhoneNumber
    from_: Union[PhoneNumber, str] = Field(..., serialization_alias='from')
    ttl: Optional[int] = Field(None, ge=300, le=259200)
    channel: ChannelType = ChannelType.MMS


class MmsImage(BaseMms):
    """Model for an MMS image message.

    Args:
        image (MmsResource): The image resource.
        to (PhoneNumber): The recipient's phone number in E.164 format. Don't use a leading plus sign.
        from_ (Union[PhoneNumber, str]): The sender's phone number in E.164 format. Don't use a leading plus sign.
        ttl (int, Optional): The duration in seconds for which the message is valid.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API will be used to send Status Webhook messages for this particular message.
    """

    image: MmsResource
    message_type: MessageType = MessageType.IMAGE


class MmsVcard(BaseMms):
    """Model for an MMS vCard message.

    Args:
        vcard (MmsResource): The vCard resource.
        to (PhoneNumber): The recipient's phone number in E.164 format. Don't use a leading plus sign.
        from_ (Union[PhoneNumber, str]): The sender's phone number in E.164 format. Don't use a leading plus sign.
        ttl (int, Optional): The duration in seconds for which the message is valid.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API will be used to send Status Webhook messages for this particular message.
    """

    vcard: MmsResource
    message_type: MessageType = MessageType.VCARD


class MmsAudio(BaseMms):
    """Model for an MMS audio message.

    Args:
        audio (MmsResource): The audio resource.
        to (PhoneNumber): The recipient's phone number in E.164 format. Don't use a leading plus sign.
        from_ (Union[PhoneNumber, str]): The sender's phone number in E.164 format. Don't use a leading plus sign.
        ttl (int, Optional): The duration in seconds for which the message is valid.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API will be used to send Status Webhook messages for this particular message.
    """

    audio: MmsResource
    message_type: MessageType = MessageType.AUDIO


class MmsVideo(BaseMms):
    """Model for an MMS video message.

    Args:
        video (MmsResource): The video resource.
        to (PhoneNumber): The recipient's phone number in E.164 format. Don't use a leading plus sign.
        from_ (Union[PhoneNumber, str]): The sender's phone number in E.164 format. Don't use a leading plus sign.
        ttl (int, Optional): The duration in seconds for which the message is valid.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API will be used to send Status Webhook messages for this particular message.
    """

    video: MmsResource
    message_type: MessageType = MessageType.VIDEO



from typing import Optional, Union

from pydantic import BaseModel, Field
from vonage_utils.types import PhoneNumber

from .base_message import BaseMessage
from .enums import ChannelType, EncodingType, MessageType


class SmsOptions(BaseModel):
    """Model for SMS options.

    Args:
        encoding_type (EncodingType, Optional): The encoding type to use for the message.
            If set to either text or unicode the specified type will be used.
            If set to auto (the default), the Messages API will automatically set
            the type based on the content.
        content_id (str, Optional): A string parameter that satisfies regulatory
            requirements when sending an SMS to specific countries. Not needed unless
            sending SMS in a country that requires a specific content ID.
        entity_id (str, Optional):  A string parameter that satisfies regulatory
            requirements when sending an SMS to specific countries. Not needed unless
            sending SMS in a country that requires a specific entity ID.
    """

    encoding_type: Optional[EncodingType] = None
    content_id: Optional[str] = None
    entity_id: Optional[str] = None


class Sms(BaseMessage):
    """Model for an SMS message.

    Args:
        to (PhoneNumber): The recipient's phone number in E.164 format.
            Don't use a leading plus sign.
        from_ (Union[PhoneNumber, str]): The sender's phone number in E.164 format.
            Don't use a leading plus sign.
        text (str): The text of the message.
        ttl (int, Optional): The duration in seconds for which the message is valid.
        sms (SmsOptions, Optional): SMS options.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API will be used to send Status Webhook messages for this particular message.
    """

    from_: Union[PhoneNumber, str] = Field(..., serialization_alias='from')
    text: str = Field(..., max_length=1000)
    ttl: Optional[int] = None
    sms: Optional[SmsOptions] = None
    channel: ChannelType = ChannelType.SMS
    message_type: MessageType = MessageType.TEXT



from typing import Optional

from pydantic import BaseModel, Field
from vonage_utils.types import PhoneNumber

from .base_message import BaseMessage
from .enums import ChannelType, MessageType


class RcsResource(BaseModel):
    """Model for a resource in an RCS message.

    Args:
        url (str): The URL of the resource.
    """

    url: str


class BaseRcs(BaseMessage):
    """Model for a base RCS message.

    Args:
        to (PhoneNumber): The recipient's phone number in E.164 format. Don't use a leading plus sign.
        from_ (str): The sender's phone number in E.164 format. Don't use a leading plus sign.
        ttl (int, Optional): The duration in seconds for which the message is valid.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API will be used to send Status Webhook messages for this particular message.
    """

    to: PhoneNumber
    from_: str = Field(..., serialization_alias='from', pattern='^[a-zA-Z0-9]+$')
    ttl: Optional[int] = Field(None, ge=300, le=259200)
    channel: ChannelType = ChannelType.RCS


class RcsText(BaseRcs):
    """Model for an RCS text message.

    Args:
        text (str): The text of the message.
        to (PhoneNumber): The recipient's phone number in E.164 format. Don't use a leading plus sign.
        from_ (str): The sender's phone number in E.164 format. Don't use a leading plus sign.
        ttl (int, Optional): The duration in seconds for which the message is valid.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API will be used to send Status Webhook messages for this particular message.
    """

    text: str = Field(..., min_length=1, max_length=3072)
    message_type: MessageType = MessageType.TEXT


class RcsImage(BaseRcs):
    """Model for an RCS image message.

    Args:
        image (RcsResource): The image resource.
        to (PhoneNumber): The recipient's phone number in E.164 format. Don't use a leading plus sign.
        from_ (str): The sender's phone number in E.164 format. Don't use a leading plus sign.
        ttl (int, Optional): The duration in seconds for which the message is valid.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API will be used to send Status Webhook messages for this particular message.
    """

    image: RcsResource
    message_type: MessageType = MessageType.IMAGE


class RcsVideo(BaseRcs):
    """Model for an RCS video message.

    Args:
        video (RcsResource): The video resource.
        to (PhoneNumber): The recipient's phone number in E.164 format. Don't use a leading plus sign.
        from_ (str): The sender's phone number in E.164 format. Don't use a leading plus sign.
        ttl (int, Optional): The duration in seconds for which the message is valid.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API will be used to send Status Webhook messages for this particular message.
    """

    video: RcsResource
    message_type: MessageType = MessageType.VIDEO


class RcsFile(BaseRcs):
    """Model for an RCS file message.

    Args:
        file (RcsResource): The file resource.
        to (PhoneNumber): The recipient's phone number in E.164 format. Don't use a leading plus sign.
        from_ (str): The sender's phone number in E.164 format. Don't use a leading plus sign.
        ttl (int, Optional): The duration in seconds for which the message is valid.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API will be used to send Status Webhook messages for this particular message.
    """

    file: RcsResource
    message_type: MessageType = MessageType.FILE


class RcsCustom(BaseRcs):
    """Model for an RCS custom message.

    Args:
        custom (dict): The custom message data.
        to (PhoneNumber): The recipient's phone number in E.164 format. Don't use a leading plus sign.
        from_ (str): The sender's phone number in E.164 format. Don't use a leading plus sign.
        ttl (int, Optional): The duration in seconds for which the message is valid.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API will be used to send Status Webhook messages for this particular message.
    """

    custom: dict
    message_type: MessageType = MessageType.CUSTOM



from typing import Optional

from pydantic import BaseModel, Field
from vonage_utils.types import PhoneNumber

from .enums import WebhookVersion


class BaseMessage(BaseModel):
    """Model with base properties for a message.

    Args:
        to (PhoneNumber): The recipient's phone number in E.164 format. Don't use a leading plus sign.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API will be used to send Status Webhook messages for this particular message.
    """

    to: PhoneNumber
    client_ref: Optional[str] = Field(None, max_length=100)
    webhook_url: Optional[str] = None
    webhook_version: Optional[WebhookVersion] = None



from enum import Enum


class MessageType(str, Enum):
    """The type of message."""

    TEXT = 'text'
    IMAGE = 'image'
    AUDIO = 'audio'
    VIDEO = 'video'
    FILE = 'file'
    TEMPLATE = 'template'
    STICKER = 'sticker'
    CUSTOM = 'custom'
    VCARD = 'vcard'


class ChannelType(str, Enum):
    """The channel used to send a message."""

    SMS = 'sms'
    MMS = 'mms'
    RCS = 'rcs'
    WHATSAPP = 'whatsapp'
    MESSENGER = 'messenger'
    VIBER = 'viber_service'


class WebhookVersion(str, Enum):
    """Which version of the Messages API will be used to send Status Webhook messages."""

    V0_1 = 'v0.1'
    V1 = 'v1'


class EncodingType(str, Enum):
    TEXT = 'text'
    UNICODE = 'unicode'
    AUTO = 'auto'



from typing import Literal, Optional

from pydantic import BaseModel, Field, model_validator

from .base_message import BaseMessage
from .enums import ChannelType, MessageType


class MessengerResource(BaseModel):
    """Model for a resource in a Messenger message.

    Args:
        url (str): The URL of the resource.
    """

    url: str


class MessengerOptions(BaseModel):
    """Model for Messenger options.

    Args:
        category (str, Optional): The category of the message. The use of different category tags enables the business to send messages for different use cases.
        tag (str, Optional): A tag describing the type and relevance of the 1:1 communication between your app and the end user.
    """

    category: Optional[Literal['response', 'update', 'message_tag']] = None
    tag: Optional[str] = None

    @model_validator(mode='after')
    def check_tag_if_category_message_tag(self):
        if self.category == 'message_tag' and not self.tag:
            raise ValueError('"tag" is required when "category" == "message_tag"')
        return self


class BaseMessenger(BaseMessage):
    """Model for a base Messenger message.

    Args:
        to (str): The ID of the message recipient.
        from_ (str): The ID of the message sender.
        messenger (MessengerOptions, Optional): Messenger options.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API will be used to send Status Webhook messages for this particular message.
    """

    to: str = Field(..., min_length=1, max_length=50)
    from_: str = Field(..., min_length=1, max_length=50, serialization_alias='from')
    messenger: Optional[MessengerOptions] = None
    channel: ChannelType = ChannelType.MESSENGER


class MessengerText(BaseMessenger):
    """Model for a Messenger text message.

    Args:
        text (str): The text of the message.
        to (str): The ID of the message recipient.
        from_ (str): The ID of the message sender.
        messenger (MessengerOptions, Optional): Messenger options.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API will be used to send Status Webhook messages for this particular message.
    """

    text: str = Field(..., max_length=640)
    message_type: MessageType = MessageType.TEXT


class MessengerImage(BaseMessenger):
    """Model for a Messenger image message.

    Args:
        image (MessengerResource): The image resource.
        to (str): The ID of the message recipient.
        from_ (str): The ID of the message sender.
        messenger (MessengerOptions, Optional): Messenger options.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API will be used to send Status Webhook messages for this particular message.
    """

    image: MessengerResource
    message_type: MessageType = MessageType.IMAGE


class MessengerAudio(BaseMessenger):
    """Model for a Messenger audio message.

    Args:
        audio (MessengerResource): The audio resource.
        to (str): The ID of the message recipient.
        from_ (str): The ID of the message sender.
        messenger (MessengerOptions, Optional): Messenger options.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API will be used to send Status Webhook messages for this particular message.
    """

    audio: MessengerResource
    message_type: MessageType = MessageType.AUDIO


class MessengerVideo(BaseMessenger):
    """Model for a Messenger video message.

    Args:
        video (MessengerResource): The video resource.
        to (str): The ID of the message recipient.
        from_ (str): The ID of the message sender.
        messenger (MessengerOptions, Optional): Messenger options.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API will be used to send Status Webhook messages for this particular message.
    """

    video: MessengerResource
    message_type: MessageType = MessageType.VIDEO


class MessengerFile(BaseMessenger):
    """Model for a Messenger file message.

    Args:
        file (MessengerResource): The file resource.
        to (str): The ID of the message recipient.
        from_ (str): The ID of the message sender.
        messenger (MessengerOptions, Optional): Messenger options.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API will be used to send Status Webhook messages for this particular message.
    """

    file: MessengerResource
    message_type: MessageType = MessageType.FILE



from typing import Literal, Optional, Union

from pydantic import BaseModel, ConfigDict, Field
from vonage_utils.types import PhoneNumber

from .base_message import BaseMessage
from .enums import ChannelType, MessageType


class WhatsappContext(BaseModel):
    """Model for the context of a WhatsApp message. This is used for quoting/replying.

    /reacting to a specific message in a conversation. When used for quoting or replying,
    the WhatsApp UI will display the new message along with a contextual bubble that
    displays the quoted/replied to message's content. When used for reacting, the WhatsApp
    UI will display the reaction emoji below the reacted to message.

    Args:
        message_uuid (str): The UUID of the message to quote/reply/react to.
    """

    message_uuid: str


class BaseWhatsapp(BaseMessage):
    """Model for a base WhatsApp message.

    Args:
        to (PhoneNumber): The recipient's phone number in E.164 format. Don't use a
            leading plus sign.
        from_ (Union[PhoneNumber, str]): The sender's phone number in E.164 format.
            Don't use a leading plus sign.
        context (WhatsappContext, Optional): Used for quoting/replying/reacting to a
            specific message in a conversation. When used for quoting or replying,
            the WhatsApp UI will display the new message along with a contextual bubble
            that displays the quoted/replied to message's content. When used for
            reacting, the WhatsApp UI will display the reaction emoji below the reacted
            to message.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be
            sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API
            will be used to send Status Webhook messages for this particular message.
    """

    from_: Union[PhoneNumber, str] = Field(..., serialization_alias='from')
    context: Optional[WhatsappContext] = None
    channel: ChannelType = ChannelType.WHATSAPP


class WhatsappText(BaseWhatsapp):
    """Model for a WhatsApp text message.

    Args:
        text (str): The text of the message.
        to (PhoneNumber): The recipient's phone number in E.164 format. Don't use a
            leading plus sign.
        from_ (Union[PhoneNumber, str]): The sender's phone number in E.164 format.
            Don't use a leading plus sign.
        context (WhatsappContext, Optional): Used for quoting/replying/reacting to a
            specific message in a conversation. When used for quoting or replying,
            the WhatsApp UI will display the new message along with a contextual bubble
            that displays the quoted/replied to message's content. When used for
            reacting, the WhatsApp UI will display the reaction emoji below the reacted
            to message.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be
            sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API
            will be used to send Status Webhook messages for this particular message.
    """

    text: str = Field(..., max_length=4096)
    message_type: MessageType = MessageType.TEXT


class WhatsappImageResource(BaseModel):
    """Model for an image attachment in a WhatsApp message.

    Args:
        url (str): The publicly accessible URL of the image attachment.
        caption (Optional[str]): Additional text to accompany the image.
    """

    url: str
    caption: Optional[str] = Field(None, min_length=1, max_length=3000)


class WhatsappImage(BaseWhatsapp):
    """Model for a WhatsApp image message.

    Args:
        image (WhatsappImageResource): The image attachment.
        to (PhoneNumber): The recipient's phone number in E.164 format. Don't use a
            leading plus sign.
        from_ (Union[PhoneNumber, str]): The sender's phone number in E.164 format.
            Don't use a leading plus sign.
        context (WhatsappContext, Optional): Used for quoting/replying/reacting to a
            specific message in a conversation. When used for quoting or replying,
            the WhatsApp UI will display the new message along with a contextual bubble
            that displays the quoted/replied to message's content. When used for
            reacting, the WhatsApp UI will display the reaction emoji below the reacted
            to message.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be
            sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API
            will be used to send Status Webhook messages for this particular message.
    """

    image: WhatsappImageResource
    message_type: MessageType = MessageType.IMAGE


class WhatsappAudioResource(BaseModel):
    """Model for an audio attachment in a WhatsApp message.

    Args:
        url (str): The publicly accessible URL of the audio attachment.
    """

    url: str = Field(..., min_length=10, max_length=2000)


class WhatsappAudio(BaseWhatsapp):
    """Model for a WhatsApp audio message.

    Args:
        audio (WhatsappAudioResource): The audio attachment.
        to (PhoneNumber): The recipient's phone number in E.164 format. Don't use a
            leading plus sign.
        from_ (Union[PhoneNumber, str]): The sender's phone number in E.164 format.
            Don't use a leading plus sign.
        context (WhatsappContext, Optional): Used for quoting/replying/reacting to a
            specific message in a conversation. When used for quoting or replying,
            the WhatsApp UI will display the new message along with a contextual bubble
            that displays the quoted/replied to message's content. When used for
            reacting, the WhatsApp UI will display the reaction emoji below the reacted
            to message.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be
            sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API
            will be used to send Status Webhook messages for this particular message.
    """

    audio: WhatsappAudioResource
    message_type: MessageType = MessageType.AUDIO


class WhatsappVideoResource(BaseModel):
    """Model for a video attachment in a WhatsApp message.

    Args:
        url (str): The publicly accessible URL of the video attachment.
        caption (Optional[str]): Additional text to accompany the video.
    """

    url: str
    caption: Optional[str] = None


class WhatsappVideo(BaseWhatsapp):
    """Model for a WhatsApp video message.

    Args:
        video (WhatsappVideoResource): The video attachment.
        to (PhoneNumber): The recipient's phone number in E.164 format. Don't use a
            leading plus sign.
        from_ (Union[PhoneNumber, str]): The sender's phone number in E.164 format.
            Don't use a leading plus sign.
        context (WhatsappContext, Optional): Used for quoting/replying/reacting to a
            specific message in a conversation. When used for quoting or replying,
            the WhatsApp UI will display the new message along with a contextual bubble
            that displays the quoted/replied to message's content. When used for
            reacting, the WhatsApp UI will display the reaction emoji below the reacted
            to message.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be
            sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API
            will be used to send Status Webhook messages for this particular message.
    """

    video: WhatsappVideoResource
    message_type: MessageType = MessageType.VIDEO


class WhatsappFileResource(BaseModel):
    """Model for a file attachment in a WhatsApp message.

    Args:
        url (str): The publicly accessible URL of the file attachment.
        caption (Optional[str]): Additional text to accompany the file.
        name (Optional[str]): Optional parameter that specifies the name of the file
            being sent. If not included, the value for `caption` will be used as the
            file name. If neither `name` or `caption` are included, the file name will be
            parsed from the url.
    """

    url: str
    caption: Optional[str] = None
    name: Optional[str] = None


class WhatsappFile(BaseWhatsapp):
    """Model for a WhatsApp file message.

    Args:
        file (WhatsappFileResource): The file attachment.
        to (PhoneNumber): The recipient's phone number in E.164 format. Don't use a
            leading plus sign.
        from_ (Union[PhoneNumber, str]): The sender's phone number in E.164 format.
            Don't use a leading plus sign.
        context (WhatsappContext, Optional): Used for quoting/replying/reacting to a
            specific message in a conversation. When used for quoting or replying,
            the WhatsApp UI will display the new message along with a contextual bubble
            that displays the quoted/replied to message's content. When used for
            reacting, the WhatsApp UI will display the reaction emoji below the reacted
            to message.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be
            sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API
            will be used to send Status Webhook messages for this particular message.
    """

    file: WhatsappFileResource
    message_type: MessageType = MessageType.FILE


class WhatsappTemplateResource(BaseModel):
    """Model for a WhatsApp template message.

    Args:
        name (str): The name of the template. For WhatsApp use your WhatsApp namespace
            (available via Facebook Business Manager), followed by a colon : and the
            name of the template to use.
        parameters (Optional[list[str]]): The parameters to be used in the template.
            An array of strings, with the first string being used for 1 in the template,
            the second being 2, etc. Only required if the template specified by name
            contains parameters.
    """

    name: str
    parameters: Optional[list[str]] = None

    model_config = ConfigDict(extra='allow')


class WhatsappTemplateSettings(BaseModel):
    """Model for WhatsApp template settings.

    Args:
        locale (Optional[str]): The BCP 47 language of the template.
        policy (Optional[Literal['deterministic']]): Policy for resolving what language
            template to use. As of now, the only valid choice is deterministic.
    """

    locale: Optional[str] = 'en_US'
    policy: Optional[Literal['deterministic']] = None


class WhatsappTemplate(BaseWhatsapp):
    """Model for a WhatsApp template message.

    Args:
        template (WhatsappTemplateResource): The template to use.
        whatsapp (WhatsappTemplateSettings): WhatsApp template settings.
        to (PhoneNumber): The recipient's phone number in E.164 format. Don't use a
            leading plus sign.
        from_ (Union[PhoneNumber, str]): The sender's phone number in E.164 format.
            Don't use a leading plus sign.
        context (WhatsappContext, Optional): Used for quoting/replying/reacting to a
            specific message in a conversation. When used for quoting or replying,
            the WhatsApp UI will display the new message along with a contextual bubble
            that displays the quoted/replied to message's content. When used for
            reacting, the WhatsApp UI will display the reaction emoji below the reacted
            to message.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be
            sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API
            will be used to send Status Webhook messages for this particular message.
    """

    template: WhatsappTemplateResource
    whatsapp: WhatsappTemplateSettings = WhatsappTemplateSettings()
    message_type: MessageType = MessageType.TEMPLATE


class WhatsappStickerUrl(BaseModel):
    """Model for a sticker attachment in a WhatsApp message.

    Args:
        url (str): The publicly accessible URL of the sticker attachment.
    """

    url: str


class WhatsappStickerId(BaseModel):
    """Model for a sticker attachment in a WhatsApp message.

    Args:
        id (str): The id of the sticker in relation to a specific WhatsApp deployment.
    """

    id: str


class WhatsappSticker(BaseWhatsapp):
    """Model for a WhatsApp sticker message.

    Args:
        sticker (Union[WhatsappStickerUrl, WhatsappStickerId]): The sticker attachment.
        to (PhoneNumber): The recipient's phone number in E.164 format. Don't use a
            leading plus sign.
        from_ (Union[PhoneNumber, str]): The sender's phone number in E.164 format.
            Don't use a leading plus sign.
        context (WhatsappContext, Optional): Used for quoting/replying/reacting to a
            specific message in a conversation. When used for quoting or replying,
            the WhatsApp UI will display the new message along with a contextual bubble
            that displays the quoted/replied to message's content. When used for
            reacting, the WhatsApp UI will display the reaction emoji below the reacted
            to message.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be
            sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API
            will be used to send Status Webhook messages for this particular message.
    """

    sticker: Union[WhatsappStickerUrl, WhatsappStickerId]
    message_type: MessageType = MessageType.STICKER


class WhatsappCustom(BaseWhatsapp):
    """Model for a WhatsApp custom message.

    Args:
        custom (dict): A custom payload, which is passed directly to WhatsApp for certain
            features such as templates and interactive messages. The schema of a custom
            object can vary widely.
        to (PhoneNumber): The recipient's phone number in E.164 format. Don't use a
            leading plus sign.
        from_ (Union[PhoneNumber, str]): The sender's phone number in E.164 format.
            Don't use a leading plus sign.
        context (WhatsappContext, Optional): Used for quoting/replying/reacting to a
            specific message in a conversation. When used for quoting or replying,
            the WhatsApp UI will display the new message along with a contextual bubble
            that displays the quoted/replied to message's content. When used for
            reacting, the WhatsApp UI will display the reaction emoji below the reacted
            to message.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be
            sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API
            will be used to send Status Webhook messages for this particular message.
    """

    custom: Optional[dict] = None
    message_type: MessageType = MessageType.CUSTOM



from typing import Literal, Optional

from pydantic import BaseModel, Field, field_validator

from .base_message import BaseMessage
from .enums import ChannelType, MessageType


class ViberAction(BaseModel):
    """Model for an action button in a Viber message.

    Args:
        url (str): A URL which is requested when the action button is clicked.
        text (str): Text which is rendered on the action button.
    """

    url: str
    text: str = Field(..., max_length=30)


class ViberOptions(BaseModel):
    """Model for Viber message options.

    Args:
        category (Literal['transaction', 'promotion'], Optional): The use of different
            category tags enables the business to send messages for different use cases.
            For Viber Business Messages the first message sent from a business to a user
            must be personal, informative and a targeted message - not promotional.
        ttl (int, Optional): The duration in seconds for which the message is valid.
        type (Literal['string', 'template'], Optional): The type of the message. To use
            "template", please contact your Vonage Account Manager to setup your templates.
    """

    category: Literal['transaction', 'promotion'] = None
    ttl: Optional[int] = Field(None, ge=30, le=259200)
    type: Optional[Literal['string', 'template']] = None


class BaseViber(BaseMessage):
    """Model for a base Viber message.

    Args:
        to (str): The recipient's phone number in E.164 format. Don't use a leading
            plus sign.
        from_ (str): The sender's phone number in E.164 format. Don't use a leading
            plus sign.
        viber_service (ViberOptions, Optional): Viber message options.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be
            sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API
            will be used to send Status Webhook messages for this particular message.
    """

    from_: str = Field(..., min_length=1, max_length=50, serialization_alias='from')
    viber_service: Optional[ViberOptions] = None
    channel: ChannelType = ChannelType.VIBER


class ViberTextOptions(ViberOptions):
    """Model for Viber text message options.

    Args:
        action (ViberAction, Optional): An action button to include in the message.
        category (Literal['transaction', 'promotion'], Optional): The use of different
            category tags enables the business to send messages for different use cases.
            For Viber Business Messages the first message sent from a business to a user
            must be personal, informative and a targeted message - not promotional.
        ttl (int, Optional): The duration in seconds for which the message is valid.
        type (Literal['string', 'template'], Optional): The type of the message. To use
            "template", please contact your Vonage Account Manager to setup your templates.
    """

    action: Optional[ViberAction] = None


class ViberText(BaseViber):
    """Model for a Viber text message.

    Args:
        text (str): The text of the message.
        to (str): The recipient's phone number in E.164 format. Don't use a leading
            plus sign.
        from_ (str): The sender's phone number in E.164 format. Don't use a leading
            plus sign.
        viber_service (ViberTextOptions, Optional): Viber text message options.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be
            sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API
            will be used to send Status Webhook messages for this particular message.
    """

    text: str = Field(..., max_length=1000)
    viber_service: Optional[ViberTextOptions] = None
    message_type: MessageType = MessageType.TEXT


class ViberImageResource(BaseModel):
    """Model for an image resource in a Viber message.

    Args:
        url (str): The URL of the image.
        caption (str, Optional): Additional text to accompany the image.
    """

    url: str
    caption: Optional[str] = None


class ViberImageOptions(ViberOptions):
    """Model for Viber image message options.

    Args:
        action (ViberAction, Optional): An action button to include in the message.
        category (Literal['transaction', 'promotion'], Optional): The use of different
            category tags enables the business to send messages for different use cases.
            For Viber Business Messages the first message sent from a business to a user
            must be personal, informative and a targeted message - not promotional.
        ttl (int, Optional): The duration in seconds for which the message is valid.
        type (Literal['string', 'template'], Optional): The type of the message. To use
            "template", please contact your Vonage Account Manager to setup your templates.
    """

    action: Optional[ViberAction] = None


class ViberImage(BaseViber):
    """Model for a Viber image message.

    Args:
        image (ViberImageResource): The image resource.
        to (str): The recipient's phone number in E.164 format. Don't use a leading
            plus sign.
        from_ (str): The sender's phone number in E.164 format. Don't use a leading
            plus sign.
        viber_service (ViberImageOptions, Optional): Viber image message options.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be
            sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API
            will be used to send Status Webhook messages for this particular message.
    """

    image: ViberImageResource
    viber_service: Optional[ViberImageOptions] = None
    message_type: MessageType = MessageType.IMAGE


class ViberVideoResource(BaseModel):
    """Model for a video resource in a Viber message.

    Args:
        url (str): The URL of the video.
        thumb_url (str): The URL of a thumbnail image to display before the video is
            played.
        caption (str, Optional): Additional text to accompany the video.
    """

    url: str
    thumb_url: str = Field(..., max_length=1000)
    caption: Optional[str] = Field(None, max_length=1000)


class ViberVideoOptions(ViberOptions):
    """Model for Viber video message options.

    Args:
        duration (str): The duration of the video in seconds.
        file_size (str): The size of the video file in MB.
        action (ViberAction, Optional): An action button to include in the message.
        category (Literal['transaction', 'promotion'], Optional): The use of different
            category tags enables the business to send messages for different use cases.
            For Viber Business Messages the first message sent from a business to a user
            must be personal, informative and a targeted message - not promotional.
        ttl (int, Optional): The duration in seconds for which the message is valid.
        type (Literal['string', 'template'], Optional): The type of the message. To use
            "template", please contact your Vonage Account Manager to setup your templates.
    """

    duration: str
    file_size: str

    @field_validator('duration')
    @classmethod
    def validate_duration(cls, value):
        value_int = int(value)
        if not 1 <= value_int <= 600:
            raise ValueError('"Duration" must be a number between 1 and 600.')
        return value

    @field_validator('file_size')
    @classmethod
    def validate_file_size(cls, value):
        value_int = int(value)
        if not 1 <= value_int <= 200:
            raise ValueError('"File size" must be a number between 1 and 200.')
        return value


class ViberVideo(BaseViber):
    """Model for a Viber video message.

    Args:
        video (ViberVideoResource): The video resource.
        to (str): The recipient's phone number in E.164 format. Don't use a leading
            plus sign.
        from_ (str): The sender's phone number in E.164 format. Don't use a leading
            plus sign.
        viber_service (ViberVideoOptions, Optional): Viber video message options.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be
            sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API
            will be used to send Status Webhook messages for this particular message.
    """

    video: ViberVideoResource
    viber_service: ViberVideoOptions
    message_type: MessageType = MessageType.VIDEO


class ViberFileResource(BaseModel):
    """Model for a file resource in a Viber message.

    Args:
        url (str): The URL for the file attachment or the path for the location of the
            file attachment. If name is included, can just be the path. If `name` is not
            included, must include the filename and extension.
        name (str, Optional): The name and extension of the file.
    """

    url: str
    name: Optional[str] = Field(None, max_length=25)


class ViberFileOptions(ViberOptions):
    """Model for Viber file message options.

    Args:
        category (Literal['transaction', 'promotion'], Optional): The use of different
            category tags enables the business to send messages for different use cases.
            For Viber Business Messages the first message sent from a business to a user
            must be personal, informative and a targeted message - not promotional.
        ttl (int, Optional): The duration in seconds for which the message is valid.
        type (Literal['string', 'template'], Optional): The type of the message. To use
            "template", please contact your Vonage Account Manager to setup your templates.
    """


class ViberFile(BaseViber):
    """Model for a Viber file message.

    Args:
        file (ViberFileResource): The file resource.
        to (str): The recipient's phone number in E.164 format. Don't use a leading
            plus sign.
        from_ (str): The sender's phone number in E.164 format. Don't use a leading
            plus sign.
        viber_service (ViberFileOptions, Optional): Viber file message options.
        client_ref (str, Optional): An optional client reference.
        webhook_url (str, Optional): The URL to which Status Webhook messages will be
            sent for this particular message.
        webhook_version (WebhookVersion, Optional): Which version of the Messages API
            will be used to send Status Webhook messages for this particular message.
    """

    file: ViberFileResource
    viber_service: Optional[ViberFileOptions] = None
    message_type: MessageType = MessageType.FILE
